@@ -60,6 +60,10 @@ static int drivetop = -1;
 /* ts A80410 : in init.c */
 extern int burn_support_untested_profiles;
 
+/* ts B10312 : in init.c */
+extern int burn_drive_role_4_allowed;
+
+
 /* ts A61021 : the unspecific part of sg.c:enumerate_common()
 */
 int burn_setup_drive(struct burn_drive *d, char *fname)
@@ -74,6 +78,7 @@ int burn_setup_drive(struct burn_drive *d, char *fname)
 	d->status = BURN_DISC_UNREADY;
 	d->do_stream_recording = 0;
         d->stream_recording_start= 0;
+	d->role_5_nwa = 0;
 	return 1;
 }
 
@@ -308,8 +313,9 @@ int burn_drive_inquire_media(struct burn_drive *d)
 int burn_drive_grab(struct burn_drive *d, int le)
 {
 	int errcode;
-	/* ts A61125 - A61202 */
-	int ret, sose;
+	/* ts A61125 - B10314 */
+	int ret, sose, stat_ret = -1;
+	struct stat stbuf;
 
 	if (!d->released) {
 		burn_print(1, "can't grab - already grabbed\n");
@@ -317,9 +323,24 @@ int burn_drive_grab(struct burn_drive *d, int le)
 	}
 	if(d->drive_role != 1) {
 		d->released = 0;
+		d->current_profile = 0xffff;
+		if (d->devname[0])
+			stat_ret = stat(d->devname, &stbuf);
 		if(d->drive_role == 2 || d->drive_role == 3) {
 			d->status = BURN_DISC_BLANK;
-			d->current_profile = 0xffff;
+		} else if(d->drive_role == 4) {
+			if (d->media_read_capacity > 0)
+				d->status = BURN_DISC_FULL;
+			else
+				d->status = BURN_DISC_EMPTY;
+		} else if(d->drive_role == 5) {
+			if (stat_ret != -1 && S_ISREG(stbuf.st_mode) &&
+			    stbuf.st_size > 0) {
+				d->status = BURN_DISC_APPENDABLE;
+				d->role_5_nwa = stbuf.st_size / 2048 +
+						!!(stbuf.st_size % 2048);
+			} else
+				d->status = BURN_DISC_BLANK;
 		} else {
 			d->status = BURN_DISC_EMPTY;
 			d->current_profile = 0;
@@ -616,13 +637,29 @@ void burn_wait_all(void)
 
 void burn_disc_erase_sync(struct burn_drive *d, int fast)
 {
-/* ts A60924 : libburn/message.c gets obsoleted
-	burn_message_clear_queue();
-*/
+	int ret;
 
 	burn_print(1, "erasing drive %s %s\n", d->idata->vendor,
 		   d->idata->product);
 
+	if (d->drive_role == 5) { /* Random access write-only drive */
+		ret = truncate(d->devname, (off_t) 0);
+		if (ret == -1) {
+			libdax_msgs_submit(libdax_messenger, -1,
+			     0x00020182,
+			     LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+			     "Cannot truncate disk file for pseudo blanking",
+			     0, 0);
+			return;
+		}
+		d->role_5_nwa = 0;
+		d->cancel = 0;
+		d->status = BURN_DISC_BLANK;
+		d->busy = BURN_DRIVE_IDLE;
+		d->progress.sector = 0x10000;
+		return;
+	}
+
 	d->cancel = 0;
 
 #ifdef Libburn_reset_progress_asynC
@@ -1361,6 +1398,7 @@ int burn_drive__fd_from_special_adr(char *adr)
 }
 
 /* @param flag bit0= accept read-only files and return 2 in this case
+               bit1= accept write-only files and return 3 in this case
 */
 static int burn_drive__is_rdwr(char *fname, int *stat_ret, 
                                struct stat *stbuf_ret,
@@ -1386,6 +1424,7 @@ static int burn_drive__is_rdwr(char *fname, int *stat_ret,
 				read_size = (off_t) 0x7ffffff0 * (off_t) 2048;  
 		}
 	}
+
 	if (is_rdwr && fd >= 0) {
 		getfl_ret = fcntl(fd, F_GETFL);
 
@@ -1400,6 +1439,9 @@ fprintf(stderr, "LIBBURN_DEBUG: burn_drive__is_rdwr: getfl_ret = %lX , O_RDWR =
 		if ((flag & 1) && getfl_ret != -1 &&
 		    (getfl_ret & mask) == O_RDONLY)
 			is_rdwr = 2;
+		if ((flag & 2) && getfl_ret != -1 &&
+		    (getfl_ret & mask) == O_WRONLY)
+			is_rdwr = 3;
 	}
 	if (stat_ret != NULL)
 		*stat_ret = st_ret;
@@ -1411,22 +1453,52 @@ fprintf(stderr, "LIBBURN_DEBUG: burn_drive__is_rdwr: getfl_ret = %lX , O_RDWR =
 }
 
 
+static int burn_role_by_access(char *fname, int flag)
+{
+/* We normally need _LARGEFILE64_SOURCE defined by the build system.
+   Nevertheless the system might use large address integers by default.
+*/
+#ifndef O_LARGEFILE
+#define O_LARGEFILE 0
+#endif
+	int fd;
+        
+	fd = open(fname, O_RDWR | O_LARGEFILE);
+	if (fd != -1) {
+		close(fd);
+		return 2;
+	}
+	fd = open(fname, O_RDONLY | O_LARGEFILE);
+        if (fd != -1) {
+		close(fd);
+		return 4;
+	}
+	fd = open(fname, O_WRONLY | O_LARGEFILE);
+	if (fd != -1) {
+		close(fd);
+		return 5;
+	}
+	if (flag & 1)
+		return 0;
+	return 2;
+}
+
+
 /* ts A70903 : Implements adquiration of pseudo drives */
 int burn_drive_grab_dummy(struct burn_drive_info *drive_infos[], char *fname)
 {
-	int ret = -1, role = 0;
-	int is_rdwr = 0, stat_ret;
+	int ret = -1, role = 0, fd;
+	int is_rdwr = 0, stat_ret = -1;
 	/* divided by 512 it needs to fit into a signed long integer */
 	off_t size = ((off_t) (512 * 1024 * 1024 - 1) * (off_t) 2048);
 	off_t read_size = -1;
 	struct burn_drive *d= NULL, *regd_d;
 	struct stat stbuf;
 
-	static int allow_role_3 = 1;
-
 	if (fname[0] != 0) {
+		fd = burn_drive__fd_from_special_adr(fname);
 		is_rdwr = burn_drive__is_rdwr(fname, &stat_ret, &stbuf,
-						&read_size, 1);
+						&read_size, 1 | 2);
 		if (stat_ret == -1 || is_rdwr) {
 			ret = burn_os_stdio_capacity(fname, &size);
 			if (ret == -1) {
@@ -1445,22 +1517,21 @@ int burn_drive_grab_dummy(struct burn_drive_info *drive_infos[], char *fname)
 				return 0;
 			}
 			if (fname[0] != 0) {
-
-				/* >>> as soon as new role 4 is introduced:
-				  if (is_rdwr == 2)  role = 4; else
-				*/
-
-				role = 2;
+				if (is_rdwr == 2 &&
+				    (burn_drive_role_4_allowed & 1))
+					role = 4;
+				else if (is_rdwr == 3 &&
+				    (burn_drive_role_4_allowed & 1))
+					role = 5;
+				else
+					role = 2;
+				if (stat_ret != -1 && role == 2 && fd == -1 &&
+				    (burn_drive_role_4_allowed & 3) == 3)
+					role = burn_role_by_access(fname,
+					    !!(burn_drive_role_4_allowed & 4));
 			} else
 				role = 0;
 		} else {
-			if(S_ISDIR(stbuf.st_mode) || !allow_role_3) {
-				libdax_msgs_submit(libdax_messenger, -1,
-				 0x00020149,
-				 LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
-				 "Unsuitable filetype for pseudo-drive", 0, 0);
-				return 0;
-			}
 			role = 3;
 		}
 	}
@@ -1481,22 +1552,39 @@ int burn_drive_grab_dummy(struct burn_drive_info *drive_infos[], char *fname)
 	}
 	free((char *) d); /* all sub pointers have been copied to *regd_d */
 	d = regd_d;
-	if (d->drive_role == 2 || d->drive_role == 3) {
-		d->status = BURN_DISC_BLANK;
+	if (d->drive_role >= 2 && d->drive_role <= 5) {
+		if (d->drive_role == 4) {
+			if (read_size > 0)
+				d->status = BURN_DISC_FULL;
+			else
+				d->status = BURN_DISC_EMPTY;
+			d->block_types[BURN_WRITE_TAO] = 0;
+			d->block_types[BURN_WRITE_SAO] = 0;
+		} else {
+			if (d->drive_role == 5 && stat_ret != -1 &&
+			    S_ISREG(stbuf.st_mode) && stbuf.st_size > 0 &&
+			    (burn_drive_role_4_allowed & 8)) {
+				d->status = BURN_DISC_APPENDABLE;
+				d->block_types[BURN_WRITE_SAO] = 0;
+				d->role_5_nwa = stbuf.st_size / 2048 +
+						!!(stbuf.st_size % 2048);
+			} else {
+				d->status = BURN_DISC_BLANK;
+				d->block_types[BURN_WRITE_SAO] =
+								BURN_BLOCK_SAO;
+				d->role_5_nwa = 0;
+			}
+			d->block_types[BURN_WRITE_TAO] = BURN_BLOCK_MODE1;
+		}
 		d->current_profile = 0xffff; /* MMC for non-compliant drive */
 		strcpy(d->current_profile_text,"stdio file");
 		d->current_is_cd_profile = 0;
 		d->current_is_supported_profile = 1;
-		d->block_types[BURN_WRITE_TAO] = BURN_BLOCK_MODE1;
-		d->block_types[BURN_WRITE_SAO] = BURN_BLOCK_SAO;
 		if (read_size >= 0)
 			/* despite its name : last valid address, not size */
 			d->media_read_capacity =
 				read_size / 2048 - !(read_size % 2048);
 		burn_drive_set_media_capacity_remaining(d, size);
-
-		/* >>> ? open file for a test ? (>>> beware of "-" = stdin) */;
-
 	} else
 		d->current_profile = 0; /* Drives return this if empty */
 
@@ -1515,6 +1603,14 @@ int burn_drive_grab_dummy(struct burn_drive_info *drive_infos[], char *fname)
 		strcpy((*drive_infos)[0].vendor,"YOYODYNE");
 		strcpy((*drive_infos)[0].product,"BLACKHOLE");
 		strcpy((*drive_infos)[0].revision,"FX02");
+	} else if (d->drive_role == 4) {
+		strcpy((*drive_infos)[0].vendor,"YOYODYNE");
+		strcpy((*drive_infos)[0].product,"WARP DRIVE");
+		strcpy((*drive_infos)[0].revision,"FX03");
+	} else if (d->drive_role == 5) {
+		strcpy((*drive_infos)[0].vendor,"YOYODYNE");
+		strcpy((*drive_infos)[0].product,"WARP DRIVE");
+		strcpy((*drive_infos)[0].revision,"FX04");
 	} else {
 		strcpy((*drive_infos)[0].vendor,"FERENGI");
 		strcpy((*drive_infos)[0].product,"VAPORWARE");
@@ -2146,6 +2242,11 @@ int burn_disc_track_lba_nwa(struct burn_drive *d, struct burn_write_opts *o,
 		return -1;
 	}
 	*lba = *nwa = 0;
+	if (d->drive_role == 5 && trackno == 0 &&
+	    d->status == BURN_DISC_APPENDABLE) {
+		*lba = *nwa = d->role_5_nwa;
+		return 1;
+	}
 	if (d->drive_role != 1)
 		return 0;
 	if (o != NULL)
@@ -2448,7 +2549,7 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 	o->current_is_cd_profile = d->current_is_cd_profile;
         o->might_simulate = 0;
 	
-	if (d->drive_role == 0)
+	if (d->drive_role == 0 || d->drive_role == 4)
 		return 0;
 	if (d->drive_role == 2) {
 		/* stdio file drive : random access read-write */
@@ -2456,12 +2557,39 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 		size = d->media_capacity_remaining;
 		burn_os_stdio_capacity(d->devname, &size);
 		burn_drive_set_media_capacity_remaining(d, size);
+
+		/* >>> This looks wrong ! */
+		/* >>> should add file size */
 		o->start_range_high = size;
+
 		o->start_alignment = 2048; /* imposting a drive, not a file */
 		o->might_do_sao = 4;
 		o->might_do_tao = 2;
 		o->advised_write_mode = BURN_WRITE_TAO;
         	o->might_simulate = 1;
+	} else if (d->drive_role == 5) {
+		/* stdio file drive : random access write-only */
+		o->start_adr = 1;
+		size = d->media_capacity_remaining;
+		burn_os_stdio_capacity(d->devname, &size);
+		burn_drive_set_media_capacity_remaining(d, size);
+
+		/* >>> start_range_low = file size rounded to 2048 */;
+
+		/* >>> This looks wrong ! */
+		/* >>> should add file size */
+		o->start_range_high = size;
+
+		o->start_alignment = 2048; /* imposting a drive, not a file */
+		if (s == BURN_DISC_APPENDABLE) {
+			if (wt == BURN_WRITE_SAO || wt == BURN_WRITE_RAW)
+				return 0;
+			o->might_do_sao = 0;
+		} else
+			o->might_do_sao = 4;
+		o->might_do_tao = 2;
+		o->advised_write_mode = BURN_WRITE_TAO;
+        	o->might_simulate = 1;
 	} else if (d->drive_role != 1) {
 		/* stdio file drive : sequential access write-only */
 		o->might_do_sao = 4;
@@ -2710,16 +2838,20 @@ int burn_drive_equals_adr(struct burn_drive *d1, char *adr2_in, int role2)
 			fd = burn_drive__fd_from_special_adr(adr2);
 			if (fd != -1)
 				exact_role_matters = 1;
-			ret = burn_drive__is_rdwr(adr2, NULL, NULL, NULL, 1);
-
-			/* >>> as soon as new role 4 is introduced:
-			       if (ret == 2)  role2 = 4; else
-			*/
-
-			if (ret == 1)
+			ret = burn_drive__is_rdwr(adr2, NULL, NULL, NULL,
+									1 | 2);
+			if (ret == 2 && (burn_drive_role_4_allowed & 1))
+				role2 = 4;
+			else if (ret == 3 && (burn_drive_role_4_allowed & 1))
+				role2 = 5;
+			else if (ret > 0)
 				role2 = 2;
 			else
 				role2 = 3;
+			if (fd == -1 &&
+			    role2 == 2 && (burn_drive_role_4_allowed & 3) == 3)
+				role2 = burn_role_by_access(adr2,
+					    !!(burn_drive_role_4_allowed & 4));
 		}
 	}
 
