@@ -373,7 +373,7 @@ int burn_read_data(struct burn_drive *d, off_t byte_address,
 	} else if (d->drive_role == 3) {
 		libdax_msgs_submit(libdax_messenger, d->global_index,
 			0x00020151,
-			LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
+			LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
 			"Read attempt on write-only drive", 0, 0);
 		return 0;
 	}
@@ -423,22 +423,35 @@ int burn_read_data(struct burn_drive *d, off_t byte_address,
 			d->stdio_fd = fd =
 				open(d->devname, O_RDONLY | O_LARGEFILE);
 		if (fd == -1) {
-			if (errno != ENOENT  || !(flag & 2))
+			if (errno == EACCES && (flag & 2)) {
+				if (!(flag & 8))
+					libdax_msgs_submit(libdax_messenger,
+					  d->global_index, 0x00020183,
+					  LIBDAX_MSGS_SEV_WARNING,
+					  LIBDAX_MSGS_PRIO_HIGH,
+			  "Failed to open device (a pseudo-drive) for reading",
+					  errno, 0);
+			} else if (errno!= ENOENT || !(flag & 2))
 				libdax_msgs_submit(libdax_messenger,
-				  d->global_index,
-				  0x00020005,
+				  d->global_index, 0x00020005,
 				  LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 			  "Failed to open device (a pseudo-drive) for reading",
 				   errno, 0);
-			ret = 0; goto ex;
+			ret = 0;
+			if (errno == EACCES && (flag & 8))
+				ret= -2;
+			goto ex;
 		}
 		if (lseek(fd, byte_address, SEEK_SET) == -1) {
-			if (!(flag & 2))
+			if (!(flag & 2)) {
+				sprintf(msg, "Cannot address start byte %.f",
+					(double) byte_address);
 				libdax_msgs_submit(libdax_messenger,
 				  d->global_index,
 				  0x00020147,
 				  LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
-				  "Cannot address start byte", errno, 0);
+				  msg, errno, 0);
+			}
 			ret = 0; goto ex;
 		}
 	}
