@@ -1005,9 +1005,12 @@ int burn_precheck_write(struct burn_write_opts *o, struct burn_disc *disc,
 	reason_pt= reasons;
 	reasons[0] = 0;
 
-	if (d->drive_role == 0) {
-		sprintf(reasons,
-			 "DRIVE: is a virtual placeholder (null-drive)");
+	if (d->drive_role == 0 || d->drive_role == 4) {
+		if (d->drive_role == 0)
+			sprintf(reasons,
+			       "DRIVE: is a virtual placeholder (null-drive)");
+		else
+			sprintf(reasons, "DRIVE: read-only pseudo drive");
 		no_media = 1;
 		goto ex;
 	}
@@ -1024,7 +1027,7 @@ int burn_precheck_write(struct burn_write_opts *o, struct burn_disc *disc,
 	reason_pt= reasons + strlen(reasons);
 	if (d->status == BURN_DISC_UNSUITABLE)
 		goto unsuitable_profile;
-	if (d->drive_role == 2 ||
+	if (d->drive_role == 2 || d->drive_role == 5 ||
 		d->current_profile == 0x1a || d->current_profile == 0x12 ||
 		d->current_profile == 0x43) { 
 		/* DVD+RW , DVD-RAM , BD-RE, emulated drive on stdio file */
@@ -1119,11 +1122,13 @@ int burn_disc_open_track_dvd_minus_r(struct burn_write_opts *o,
 #endif
 
 	if (o->write_type == BURN_WRITE_SAO) { /* DAO */
- 		/* Round track size up to write chunk size and reserve track */
 		size = ((off_t) burn_track_get_sectors(s->track[tnum]))
 			* (off_t) 2048;
-		if (size % o->obs)
+
+ 		/* Eventually round track size up to write chunk */
+		if (o->obs_pad && (size % o->obs))
 			size += (off_t) (o->obs - (size % o->obs));
+
 		ret = d->reserve_track(d, size);
 		if (ret <= 0) {
 			sprintf(msg, "Cannot reserve track of %.f bytes",
@@ -1164,6 +1169,10 @@ int burn_disc_open_track_dvd_plus_r(struct burn_write_opts *o,
  		/* Round track size up to write chunk size and reserve track */
 		size = ((off_t) burn_track_get_sectors(s->track[tnum]))
 			* (off_t) 2048;
+		/* o->obs should be 32k or 64k already. But 32k alignment
+		  was once performed in d->reserve_track() */
+		if (o->obs % 32768)
+			o->obs += 32768 - (o->obs % 32768);
 		if (size % o->obs)
 			size += (off_t) (o->obs - (size % o->obs));
 		ret = d->reserve_track(d, size);
@@ -1892,6 +1901,8 @@ int burn_dvd_write_sync(struct burn_write_opts *o,
 				  msg, 0, 0);
 			burn_track_set_size(t, default_size);
 		}
+		/* Whether to fill-up last 32k buffer of track. */
+		o->obs_pad = (o->write_type != BURN_WRITE_SAO);
 		ret = burn_disc_setup_dvd_minus_r(o, disc);
 		if (ret <= 0) {
 			sprintf(msg,
@@ -1902,8 +1913,6 @@ int burn_dvd_write_sync(struct burn_write_opts *o,
 				msg, 0, 0);
 			goto early_failure;
 		}
-		/* ??? padding needed ??? cowardly doing it for now */
-		o->obs_pad = 1; /* fill-up track's last 32k buffer */
 		
 	} else if (d->current_profile == 0x1b || d->current_profile == 0x2b ||
 		   d->current_profile == 0x41) {
@@ -2036,6 +2045,16 @@ int burn_stdio_open_write(struct burn_drive *d, off_t start_byte,
 	char msg[160];
 	off_t lseek_res;
 
+	if(d->drive_role == 4) {
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x00020181,
+			LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+			"Pseudo-drive is a read-only file. Cannot write.",
+			0, 0);
+		return 0;
+	}
+	if (d->drive_role == 5 || d->drive_role == 3)
+		mode = O_WRONLY | O_CREAT | O_LARGEFILE;
 	if (d->devname[0] == 0) /* null drives should not come here */
 		return -1;
 	fd = burn_drive__fd_from_special_adr(d->devname);
@@ -2054,7 +2073,7 @@ int burn_stdio_open_write(struct burn_drive *d, off_t start_byte,
 	} 
 	if (start_byte < 0)
 		start_byte = 0;
-	if (d->drive_role == 2) {
+	if (d->drive_role == 2 || d->drive_role == 5) {
 		lseek_res = lseek(fd, start_byte, SEEK_SET);
 		if (lseek_res == -1) {
 			sprintf(msg, "Cannot address start byte %.f",
@@ -2333,6 +2352,9 @@ int burn_stdio_write_sync(struct burn_write_opts *o,
 	/* open target file */
 	if (d->stdio_fd >= 0)
 		close(d->stdio_fd);
+	if (d->drive_role == 5 && d->status == BURN_DISC_APPENDABLE &&
+            o->start_byte < 0)
+		o->start_byte = d->role_5_nwa * 2048;
 	d->stdio_fd = burn_stdio_open_write(d, o->start_byte, 2048, 0);
 	if (d->stdio_fd == -1)
 		{ret = 0; goto ex;}
@@ -2415,6 +2437,14 @@ calloc() seems not to have the desired effect. valgrind warns:
 	d->rlba = -150;
 	d->toc_temp = 9;
 
+	if(d->drive_role == 4) {
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x00020181,
+			LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+			"Pseudo-drive is a read-only file. Cannot write.",
+			0, 0);
+		goto fail_wo_sync;
+	}
 	/* ts A70904 */
 	if (d->drive_role != 1) {
 		ret = burn_stdio_write_sync(o, disc);
@@ -2638,8 +2668,16 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 			"Drive is a virtual placeholder (null-drive)", 0, 0);
 		return 0;
 	}
+	if(d->drive_role == 4) {
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x00020181,
+			LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+			"Pseudo-drive is a read-only file. Cannot write.",
+			0, 0);
+		return 0;
+	}
 
-	if(d->drive_role == 2)
+	if(d->drive_role == 2 || d->drive_role == 5)
 		alignment = 2 * 1024;
 	if (d->current_profile == 0x12) /* DVD-RAM */
 		alignment = 2 * 1024;
