@@ -469,6 +469,12 @@ struct isoburn_imgen_opts {
      */
     unsigned int joliet_longer_paths :1;
 
+    /**
+     * Allow leaf names in the Joliet tree to have up to 103 characters
+     * rather than 64.
+     */
+    unsigned int joliet_long_names :1;
+
     /**
      * Store timestamps as GMT rather than in local time.
      */
