@@ -1179,3 +1179,33 @@ int Xorriso__bourne_to_reg(char bourne_expr[], char reg_expr[], int flag)
 
 #endif /* ! Xorriso_fileliste_externaL */
 
+
+int Xorriso__hide_mode(char *mode, int flag)
+{
+ if(strcmp(mode, "on") == 0)
+   return(1 | 2);
+ else if(strcmp(mode, "iso_rr") == 0)
+   return(1);
+ else if(strcmp(mode, "joliet") == 0)
+   return(2);
+ else if(strcmp(mode, "off") == 0)
+   return(0);
+ return(-1);
+}
+
+
+char *Xorriso__hide_mode_text(int hide_mode, int flag)
+{
+ switch(hide_mode & 3) {
+ case 0:
+   return "off";
+ case 1:
+   return "iso_rr";
+ case 2:
+   return "joliet";
+ case 3:
+   return "on";
+ }
+ return "invalid";
+}
+
