@@ -300,6 +300,25 @@ int Xorriso_option_dvd_obs(struct XorrisO *xorriso, char *obs, int flag)
 }
 
 
+/* Option -early_stdio_test */
+int Xorriso_option_early_stdio_test(struct XorrisO *xorriso, char *mode,
+                                    int flag)
+{
+ if(strcmp(mode, "on") == 0)
+   xorriso->early_stdio_test= 2 | 4;
+ else if(strcmp(mode, "off") == 0)
+   xorriso->early_stdio_test= 0;
+ else if(strcmp(mode, "appendable_wo") == 0)
+   xorriso->early_stdio_test= 2 | 4 | 8;
+ else {
+   sprintf(xorriso->info_text, "-early_stdio_test: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   return(0);
+ }
+ return(1);
+}
+
+
 /* Option -eject */
 /* @param flag bit0=do not report toc of eventually remaining drives
 */
@@ -1414,6 +1433,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Enable production of scdbackup tag with -md5 on",
 "  -ban_stdio_write",
 "                 Allow for writing only the usage of optical drives.",
+"  -early_stdio_test \"on\"|\"appendable_wo\"|\"off\"",
+"                 Classify stdio drives by effective access permissions.",
 "  -blank \"fast\"|\"all\"|\"deformat\"|\"deformat_quickest\"",
 "                 Blank media resp. invalidate ISO image on media.",
 "  -format \"as_needed\"|\"full\"|\"fast\"|\"by_index_#\"|\"by_size_#\"",
@@ -1906,6 +1927,10 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              prevents reading and interpretation of startup files.",
 "  -print text",
 "              Print a text to result channel.",
+"  -print_info text",
+"              Print a text to info channel.",
+"  -print_mark text",
+"              Print a text to mark channel.",
 "  -prompt text",
 "              Wait for Enter key resp. for a line of input at stdin.",
 "  -errfile_log mode path|channel",
