@@ -461,14 +461,14 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "calm_drive","cd","cdi","cdx","charset","close",
     "commit_eject","compliance","copyright_file",
     "dev","dialog","disk_dev_ino","disk_pattern","displacement",
-    "dummy","dvd_obs","eject",
+    "dummy","dvd_obs","early_stdio_test", "eject",
     "iso_rr_pattern","follow","format","fs","gid","grow_blindly","hardlinks",
     "history","indev","in_charset","joliet",
     "list_delimiter","list_profiles","local_charset",
     "mark","md5","mount_opts","not_leaf","not_list","not_mgt",
     "options_from_file","osirrox","outdev","out_charset","overwrite",
     "pacifier","padding","path_list","pathspecs","pkt_output",
-    "preparer_id","print","prompt",
+    "preparer_id","print","print_info","print_mark","prompt",
     "prog","prog_help","publisher","quoted_not_list","quoted_path_list",
     "reassure","report_about","rom_toc_scan","scsi_log",
     "session_log","speed","split_size","status","status_history_max",
@@ -853,6 +853,10 @@ next_command:;
  } else if(strcmp(cmd,"dux")==0 || strcmp(cmd,"dusx")==0) {
    ret= Xorriso_option_lsx(xorriso, argc, argv, idx, (cmd[2]!='s')|4);
 
+ } else if(strcmp(cmd,"early_stdio_test")==0) {
+   (*idx)++;
+   ret= Xorriso_option_early_stdio_test(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"eject")==0) {
    (*idx)++;
    ret= Xorriso_option_eject(xorriso, arg1, 0);
@@ -1153,6 +1157,14 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_print(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"print_info")==0) {
+   (*idx)++;
+   ret= Xorriso_option_print(xorriso, arg1, 1);
+
+ } else if(strcmp(cmd,"print_mark")==0) {
+   (*idx)++;
+   ret= Xorriso_option_print(xorriso, arg1, 2);
+
  } else if(strcmp(cmd,"print_size")==0) {
    Xorriso_option_print_size(xorriso, 0);
 
