@@ -251,6 +251,18 @@ struct XorrisO { /* the global context of xorriso */
 
  int drives_exclusive;   /* burn_preset_device_open() param exclusive */
 
+ int early_stdio_test;   /* For burn_allow_drive_role_4():
+                            bit1= Test whether a stdio drive can be opened for
+                                  read-write resp. read-only resp. write only.
+                            bit2= Classify files which cannot be opened at all
+                                  as role 0 : useless dummy.
+                            bit3= Classify non-empty role 5 drives as
+                                  BURN_DISC_APPENDABLE with NWA after the
+                                  end of the file. It is nevertheless
+                                  possible to change this address by call
+                                  burn_write_opts_set_start_byte().
+                          */
+
  int do_calm_drive;      /* bit0= calm down drive after aquiring it */
 
  char indev[SfileadrL];
